@@ -45,12 +45,14 @@ def _get_image(img_h=300, img_w=300):
 def test_normalize():
     height, width = 300, 300
     img = _get_image(height, width)
+    img = np.expand_dims(img, axis=0)
+    img = np.expand_dims(img, axis=-1)
     normalized_img = processing.normalize(img)
     np.testing.assert_almost_equal(normalized_img.mean(), 0)
     np.testing.assert_almost_equal(normalized_img.var(), 1)
 
 
-def test_phase_preprocess():
+def test_histogram_normalization():
     height, width = 300, 300
     img = _get_image(height, width)
 
@@ -58,10 +60,14 @@ def test_phase_preprocess():
     img = np.expand_dims(img, axis=0)
     img = np.expand_dims(img, axis=-1)
 
-    preprocessed_img = processing.phase_preprocess(img)
+    preprocessed_img = processing.histogram_normalization(img)
     assert (preprocessed_img <= 1).all() and (preprocessed_img >= -1).all()
 
-    preprocessed_img = processing.phase_preprocess(img.astype('uint16'))
+    preprocessed_img = processing.histogram_normalization(img.astype('uint16'))
+    assert (preprocessed_img <= 1).all() and (preprocessed_img >= -1).all()
+
+    # test legacy version
+    preprocessed_img = processing.phase_preprocess(img)
     assert (preprocessed_img <= 1).all() and (preprocessed_img >= -1).all()
 
 
@@ -76,14 +82,14 @@ def test_pixelwise():
     channels = 4
     img = np.random.rand(1, 300, 300, channels)
     pixelwise_img = processing.pixelwise(img)
-    np.testing.assert_equal(pixelwise_img.shape, (300, 300, 1))
+    assert pixelwise_img.shape == img.shape[:-1] + (1,)
 
 
 def test_watershed():
     channels = np.random.randint(4, 8)
-    img = np.random.rand(300, 300, channels)
+    img = np.random.rand(1, 300, 300, channels)
     watershed_img = processing.watershed(img)
-    np.testing.assert_equal(watershed_img.shape, (300, 300, 1))
+    assert watershed_img.shape == img.shape[:-1] + (1,)
 
 
 def test_correct_drift():
